/*
 * GaugeImpl.java
 *
 * Created on February 15, 2002, 2:41 PM
 */

package edu.cmu.cs.able.gaugeInfrastructure;
import java.util.*;
import edu.cmu.cs.able.gaugeInfrastructure.util.*;
/** This class provides the root class for
 * all gauge implementations. It should contain
 * the processing and calculation part of the gauge.
 * It is the part of the gauge that is completely
 * independent of the transport mechanism used.
 *
 * @author Bradley Schmerl
 * @version 1.0
 */
public abstract class GaugeImpl implements  Runnable {
    
    /** The type of the gauge being implemented.
     */    
    public static String gaugeType = "edu.cmu.cs.able.gaugeInfrastructure.genericGauge";
    /** The ID of the gauge.
     */    
    protected GaugeID gaugeID = null;
    /** Indicates whether to do reporting or not.
     */    
    protected boolean doReporting = false;
    /** The thread that does periodic reporting of values.
     */    
    protected Thread reportingThread = null;
    /** The frequency to report values (in milliseconds).
     */    
    protected int reportingFrequency = 1000;
    /** The setup parameters for the gauge instance.
     */    
    protected StringPairVector setupParams;
    /** The mappings of values to architectural properties.
     */    
    protected Hashtable mappings = new Hashtable ();
    /** The bus through which values are reported.
     */    
    protected GaugeReportingBus gaugeBus;
    
    /** Creates a new instance of GaugeImpl
     * @param gid The gauge ID of the new gauge.
     * @param setupParams The setup parameters used to construct the gauge.
     * @param mappings The mappings of values to architectural properties.
     * @param bus The bus to be used for reporting.
     */
    public GaugeImpl (GaugeID gid, StringPairVector setupParams, StringPairVector mappings, GaugeReportingBus bus) {
        gaugeID = gid;
        gaugeBus = bus;
        setupParams = setupParams;
        if (mappings != null) {
            xferMappings (mappings);
        }
        this.startReportingThread ();
    }
    
    /** The method used to start the gauge reporting thread.
     */    
    public void run () {
        Thread myThread = Thread.currentThread ();
        while (reportingThread == myThread) {
            if (doReporting) this.reportNewValue ();
            try {
                Thread.sleep (reportingFrequency);
            }
            catch (InterruptedException e) {
                reportingThread = null;
            }
        }
    }
    
    /** Starts the gauge reporting thread.
     */    
    public void startReportingThread () {
        if (reportingThread == null) {
            reportingThread = new Thread (this, "reporting");
            reportingThread.start ();
        }
    }
    
    /** Stops the gauge reporting thread.
     */    
    public void stopReportingThread () {
        reportingThread = null;
    }
    
    private void xferMappings (StringPairVector mappingsVec) {
        for (int i=0; i < mappingsVec.size (); i++) {
            mappings.put (mappingsVec.nameAt (i), mappingsVec.valueAt (i));
        }
    }
    
    /** Used to check if the gauge has been successfully created.
     * Typically, need to check that all setup parameters were valid
     * and all probes have been deployed, installed, and activated
     * successfully.
     * @return true if the creation was successful, false otherwise.
     */    
    abstract public boolean consistentlyCreated ();
    /** Implemented by specific gauges to report a new value.
     */    
    abstract public void reportNewValue ();
    /** Is used to configure the gauge.
     * @param configParams The configuration parameters
     * @return true if the configuration was successful.
     */    
    abstract public boolean configure (StringPairVector configParams);
    /** Used to get the current value of all the values that the gauge reports.
     * @param values This is filled in with triples of value name, property name (what the value is mapped to), and value.
     * @return true if the values could be queried.
     */    
    abstract public boolean queryAllValues (GaugeValueVector values);
    /** Used to query the current state of the gauge.
     * @param setupParams Filled with the parameters that were used to setup the gauge.
     * @param configParams Filled with parameters indicating how the gauge is currently configured.
     * @param mappings Filled with the mappings between values and high-level properties.
     * @return true if the query was successful.
     */    
    abstract public boolean queryState (StringPairVector setupParams, StringPairVector configParams, StringPairVector mappings);
    /** Queries the value of a particular gauge value.
     * @param valueName The name of the value to be reported.
     * @return The value indicated by valueName.
     */    
    abstract public String queryValue (String valueName);
    
}
